package com.dayou.controller;

import com.dayou.annotation.IgnoreAuth;
import com.dayou.vo.FinanceClaimVO;
import com.dayou.vo.RealFundMajorStatVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.service.IFinanceClaimService;
import com.dayou.entity.FinanceClaim;
import com.dayou.common.RestResponse;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

/**
 * 实收款认领
 *
 * @author wucl
 * @since 2023-12-08
 */
@RestController
@RequestMapping("financeClaim")
@Slf4j
public class FinanceClaimController extends BaseController {
    @Autowired
    private IFinanceClaimService financeClaimService;

    /**
    * 实收款认领列表
    */
    @GetMapping("")
    public RestResponse<Page<FinanceClaim>> page(FinanceClaim financeClaim, Page page){
        Page<FinanceClaim> pages=financeClaimService.selectPage(page,financeClaim);
        return RestResponse.data(pages);
    }

    /**
     * 实收款认领详情
     */
    @GetMapping("/{id}")
    public RestResponse<FinanceClaim> detail(@PathVariable Long id){
        FinanceClaim xFinanceClaim =financeClaimService.detail(id);
        return RestResponse.data(xFinanceClaim);
     }

    /**
     * 实收款认领新增
     */
    @PostMapping("")
    public RestResponse<Boolean> save(@RequestBody @Valid FinanceClaim financeClaim) {
        Boolean ret = financeClaimService.add(financeClaim);
        return RestResponse.data(ret);
    }

    /**
     * 实收款认领更新
     */
    @PutMapping("")
    public RestResponse<Boolean> update(@RequestBody FinanceClaim financeClaim) {
        Boolean ret = financeClaimService.update(financeClaim);
        return RestResponse.data(ret);
    }

    /**
     * 实收款认领删除
     */
    @DeleteMapping("/{id}")
    public RestResponse<Boolean> delete(@PathVariable Long id) {
        Boolean ret = financeClaimService.delete(id);
        return RestResponse.data(ret);
    }

    /**
     * 某实收款认领列表
     */
    @GetMapping("/list/{id}")
    public RestResponse<List<FinanceClaimVO>> getList(@PathVariable Long id){
        List<FinanceClaimVO> list = financeClaimService.getList(id);
        return RestResponse.data(list);
    }

    /**
     * 个贷认领模板下载
     * @param request
     * @param response
     */
    @IgnoreAuth
    @GetMapping("/personal/claim/template")
    public void downloadPersonalClaimTemplate(HttpServletRequest request, HttpServletResponse response){
        try {
            XSSFWorkbook workbook = new XSSFWorkbook(new ClassPathResource("xlsx/personal/personal_claim_template.xlsx").getInputStream());
            writeXss2Response(request,response,workbook,"个贷认领模板.xlsx");
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 个贷模板认领实收款
     * @param file
     * @return
     */
    @PostMapping("/personal/temp/upload/{financeFundId}")
    public RestResponse<Boolean> personalClaimByTemplate(@RequestPart("file") MultipartFile file,@PathVariable("financeFundId") Long financeFundId){
        Boolean ret = financeClaimService.personalClaimByTemplate(file,financeFundId);
        return RestResponse.data(ret);
    }

    /**
     * 大中型实收款统计列表
     */
    @GetMapping("/major/stat")
    public RestResponse<Page<RealFundMajorStatVO>> majorStat(RealFundMajorStatVO majorStat, Page page){
        Page<RealFundMajorStatVO> pages=financeClaimService.majorStat(page,majorStat);
        return RestResponse.data(pages);
    }

    /**
     * 大中型实收款统计列表 导出
     */
    @GetMapping("/major/stat/export")
    public void majorStatExport(RealFundMajorStatVO majorStat, HttpServletResponse response) throws IOException {
        List<RealFundMajorStatVO> list = financeClaimService.majorStatExport(majorStat);
        exportPlus(response,"大中型实收款列表",list,RealFundMajorStatVO.class);
    }
}

