package com.dayou.service;
import com.dayou.dto.*;
import com.dayou.entity.AssetsProduction;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 资产产品表 服务类
 * </p>
 *
 * @author GouGengquan
 * @since 2024-03-28
 */
public interface IAssetsProductionService extends IService<AssetsProduction> {

        /**
         * 根据资产业务id查询资产业务产品
         * @param id 资产业务id
         * @return List<AssetsProduction>
         */
        List<AssetsProduction> getAroductionByAssetsId(Long id);

        Page<AssetsProduction> selectPage(Page page,AssetsProduction assetsProduction);

        AssetsProduction detail(Long id);

        Boolean add(AssetsProduction assetsProduction);

        Boolean update(AssetsProduction assetsProduction);

        Boolean delete(Long id);

        /**
         * 根据业务id获取产品信息类型集合
         * @param businessId 资产业务id
         * @return List<String>
         */
        List<String> getProductionTypes(Long businessId);

        /**
         * 根据业务id获取已选择产品
         * @param businessId 资产业务id
         * @return AssetsProduction
         */
        AssetsProduction getProductionByBusinessId(Long businessId);

        /**
         * 撰写产品信息
         * @param assetsProduction 产品信息字段
         * @return Boolean
         */
        Boolean writeAssetsProduction(AssetsProduction assetsProduction);

        /**
         * 根据业务id与产品号获取资产产品详细信息
         * @param dto 查询dto
         * @return AssetsProduction
         */
        AssetsProduction getAssetsProductionDetail(AssetsProductionDetailDTO dto);

        /**
         * 根据业务id与产品号查询产品信息
         * @param aProListSelectDTO 查询dto
         * @return List<AssetsProduction>
         */
        List<AssetsProduction> getProductionList(AProListSelectDTO aProListSelectDTO);

        /**
         * 根据业务id与产品号查询该产品是否已有提交的流程实例信息
         * @param aProListSelectDTO 查询dto
         * @return Integer
         */
        Integer getInatanceCount(AProListSelectDTO aProListSelectDTO);

        /**
         * 根据业务id与产品号集合获取每个产品是否有提交过的流程实例信息
         * @return List<Integer>
         */
        List<Integer> getInatanceCountList(AProListSelectDTO aProListSelectDTO);

        /**
         * 产品出入库
         * @param wareHouseDTO 出入库dto
         * @return Boolean
         */
        Boolean assetsProRepertory(AssetsProWareHouseDTO wareHouseDTO);

        /**
         * 根据资产产品id更新文件路由信息
         * @param assetsProduction 产品信息dto
         * @return Boolean
         */
        Boolean updateProductionFileInfo(AssetsProduction assetsProduction);

        /**
         * 获取产品实收款
         * @param productionNo 产品号
         * @return BigDecimal
         */
        BigDecimal getProductionRealAmount(String productionNo);

        /**
         * 产品标记送达
         * @param productionNo 产品号
         * @return Boolean
         */
        Boolean markDelivery(String productionNo);

}
